@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE DeriveFunctor #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE OverloadedStrings #-}
@@ -29,8 +30,17 @@ module Text.Pandoc.Class.PandocPure
   ) where
 
 import Codec.Archive.Zip
+import Control.Monad.Trans ( MonadTrans(lift) )
 import Control.Monad.Except
+    ( ExceptT(..), MonadError(throwError), runExceptT )
 import Control.Monad.State.Strict
+    ( StateT(StateT),
+      State,
+      MonadState(put, get),
+      modify,
+      evalState,
+      evalStateT )
+import Control.Monad (foldM)
 import Data.Default
 import Data.Text (Text)
 import Data.Time (UTCTime)
@@ -40,7 +50,7 @@ import Data.Word (Word8)
 import System.Directory (doesDirectoryExist, getDirectoryContents)
 import System.FilePath ((</>))
 import System.FilePath.Glob (match, compile)
-import System.Random (StdGen, split, mkStdGen)
+import System.Random (StdGen, mkStdGen)
 import Text.Pandoc.Class.CommonState (CommonState (..))
 import Text.Pandoc.Class.PandocMonad
 import Text.Pandoc.Error
@@ -50,6 +60,15 @@ import qualified Data.Map as M
 import qualified Data.Text as T
 import qualified System.Directory as Directory (getModificationTime)
 
+#if MIN_VERSION_random(1,3,0)
+import System.Random (splitGen, SplitGen)
+
+split :: SplitGen g => g -> (g, g)
+split = splitGen
+#else
+import System.Random (split)
+#endif
+
 -- | The 'PureState' contains ersatz representations
 -- of things that would normally be obtained through IO.
 data PureState = PureState
