@@ -1,4 +1,3 @@
-{-# LANGUAGE ViewPatterns      #-}
 {-# LANGUAGE TupleSections     #-}
 {-# LANGUAGE LambdaCase        #-}
 {-# LANGUAGE OverloadedStrings #-}
@@ -31,17 +30,20 @@ module Text.Pandoc.Readers.Docx.Parse ( Docx(..)
                                       , RunStyle(..)
                                       , VertAlign(..)
                                       , ParIndentation(..)
+                                      , Justification(..)
                                       , ParagraphStyle(..)
                                       , ParStyle
                                       , CharStyle(cStyleData)
                                       , Row(..)
                                       , TblHeader(..)
+                                      , Align(..)
                                       , Cell(..)
                                       , VMerge(..)
                                       , TrackedChange(..)
                                       , ChangeType(..)
                                       , ChangeInfo(..)
                                       , FieldInfo(..)
+                                      , IndexEntry(..)
                                       , Level(..)
                                       , ParaStyleName
                                       , CharStyleName
@@ -56,6 +58,7 @@ module Text.Pandoc.Readers.Docx.Parse ( Docx(..)
                                       , constructBogusParStyleData
                                       , leftBiasedMergeRunStyle
                                       , rowsToRowspans
+                                      , extractTarget
                                       ) where
 import Text.Pandoc.Readers.Docx.Parse.Styles
 import Codec.Archive.Zip
@@ -79,7 +82,7 @@ import Text.Pandoc.Shared (filteredFilesFromArchive, safeRead)
 import qualified Text.Pandoc.UTF8 as UTF8
 import Text.TeXMath (Exp)
 import Text.TeXMath.Readers.OMML (readOMML)
-import Text.TeXMath.Unicode.Fonts (Font (..), getUnicode, textToFont)
+import Text.Pandoc.Readers.Docx.Symbols (symbolMap, Font(..), textToFont)
 import Text.Pandoc.XML.Light
     ( filterChild,
       findElement,
@@ -116,7 +119,7 @@ data ReaderState = ReaderState { stateWarnings :: [T.Text]
                  deriving Show
 
 data FldCharState = FldCharOpen
-                  | FldCharFieldInfo FieldInfo
+                  | FldCharFieldInfo T.Text
                   | FldCharContent FieldInfo [ParPart]
                   deriving (Show)
 
@@ -148,13 +151,46 @@ mapD f xs =
   in
    concatMapM handler xs
 
+isAltContentRun :: NameSpaces -> Element -> Bool
+isAltContentRun ns element
+  | isElem ns "w" "r" element
+  , Just _altContentElem <- findChildByName ns "mc" "AlternateContent" element
+  = True
+  | otherwise
+  = False
+
+-- Elements such as <w:shape> are not always preferred
+-- to be unwrapped. Only if they are part of an AlternateContent
+-- element, they should be unwrapped.
+-- This strategy prevents VML images breaking.
+unwrapAlternateContentElement :: NameSpaces -> Element -> [Element]
+unwrapAlternateContentElement ns element
+  | isElem ns "mc" "AlternateContent" element
+  || isElem ns "mc" "Fallback" element
+  || isElem ns "w" "pict" element
+  || isElem ns "v" "group" element
+  || isElem ns "v" "rect" element
+  || isElem ns "v" "roundrect" element
+  || isElem ns "v" "shape" element
+  || isElem ns "v" "textbox" element
+  || isElem ns "w" "txbxContent" element
+  = concatMap (unwrapAlternateContentElement ns) (elChildren element)
+  | otherwise
+  = unwrapElement ns element
+
 unwrapElement :: NameSpaces -> Element -> [Element]
 unwrapElement ns element
   | isElem ns "w" "sdt" element
   , Just sdtContent <- findChildByName ns "w" "sdtContent" element
   = concatMap (unwrapElement ns) (elChildren sdtContent)
+  | isElem ns "w" "r" element
+  , Just alternateContentElem <- findChildByName ns "mc" "AlternateContent" element
+  = unwrapAlternateContentElement ns alternateContentElem
   | isElem ns "w" "smartTag" element
   = concatMap (unwrapElement ns) (elChildren element)
+  | isElem ns "w" "p" element
+  , Just (modified, altContentRuns) <- extractChildren element (isAltContentRun ns)
+  = (unwrapElement ns modified) ++ concatMap (unwrapElement ns) altContentRuns
   | otherwise
   = [element{ elContent = concatMap (unwrapContent ns) (elContent element) }]
 
@@ -221,29 +257,39 @@ data ChangeInfo = ChangeInfo ChangeId Author (Maybe ChangeDate)
 data TrackedChange = TrackedChange ChangeType ChangeInfo
                    deriving Show
 
-data ParagraphStyle = ParagraphStyle { pStyle      :: [ParStyle]
-                                     , indentation :: Maybe ParIndentation
-                                     , numbered    :: Bool
-                                     , dropCap     :: Bool
-                                     , pChange     :: Maybe TrackedChange
-                                     , pBidi       :: Maybe Bool
+data Justification = JustifyBoth | JustifyLeft | JustifyRight | JustifyCenter
+  deriving (Show, Eq)
+
+data ParagraphStyle = ParagraphStyle { pStyle        :: [ParStyle]
+                                     , indentation   :: Maybe ParIndentation
+                                     , justification :: Maybe Justification
+                                     , numbered      :: Bool
+                                     , dropCap       :: Bool
+                                     , pChange       :: Maybe TrackedChange
+                                     , pBidi         :: Maybe Bool
+                                     , pKeepNext     :: Bool
                                      }
                       deriving Show
 
 defaultParagraphStyle :: ParagraphStyle
 defaultParagraphStyle = ParagraphStyle { pStyle = []
                                        , indentation = Nothing
-                                       , numbered    = False
-                                       , dropCap     = False
-                                       , pChange     = Nothing
-                                       , pBidi       = Just False
+                                       , justification = Nothing
+                                       , numbered = False
+                                       , dropCap = False
+                                       , pChange = Nothing
+                                       , pBidi = Just False
+                                       , pKeepNext = False
                                        }
 
 
 data BodyPart = Paragraph ParagraphStyle [ParPart]
+              | Heading Int ParaStyleName ParagraphStyle T.Text T.Text (Maybe Level)
+                 [ParPart]
               | ListItem ParagraphStyle T.Text T.Text (Maybe Level) [ParPart]
-              | Tbl T.Text TblGrid TblLook [Row]
-              | TblCaption ParagraphStyle [ParPart]
+              | Tbl (Maybe T.Text) T.Text TblGrid TblLook [Row]
+              | Captioned ParagraphStyle [ParPart] BodyPart
+              | HRule
               deriving Show
 
 type TblGrid = [Integer]
@@ -258,7 +304,10 @@ data Row = Row TblHeader [Cell] deriving Show
 
 data TblHeader = HasTblHeader | NoTblHeader deriving (Show, Eq)
 
-data Cell = Cell GridSpan VMerge [BodyPart]
+data Align = AlignDefault | AlignLeft | AlignRight | AlignCenter
+  deriving (Show, Eq)
+
+data Cell = Cell Align GridSpan VMerge [BodyPart]
             deriving Show
 
 type GridSpan = Integer
@@ -271,7 +320,7 @@ data VMerge = Continue
 
 rowsToRowspans :: [Row] -> [[(Int, Cell)]]
 rowsToRowspans rows = let
-  removeMergedCells = fmap (filter (\(_, Cell _ vmerge _) -> vmerge == Restart))
+  removeMergedCells = fmap (filter (\(_, Cell _ _ vmerge _) -> vmerge == Restart))
   in removeMergedCells (foldr f [] rows)
   where
     f :: Row -> [[(Int, Cell)]] -> [[(Int, Cell)]]
@@ -287,9 +336,9 @@ rowsToRowspans rows = let
     g cells columnsLeftBelow (Just rowBelow) =
         case cells of
           [] -> []
-          thisCell@(Cell thisGridSpan _ _) : restOfRow -> case rowBelow of
+          thisCell@(Cell _ thisGridSpan _ _) : restOfRow -> case rowBelow of
             [] -> map (1,) cells
-            (spanSoFarBelow, Cell gridSpanBelow vmerge _) : _ ->
+            (spanSoFarBelow, Cell _ gridSpanBelow vmerge _) : _ ->
               let spanSoFar = case vmerge of
                     Restart -> 1
                     Continue -> 1 + spanSoFarBelow
@@ -299,7 +348,7 @@ rowsToRowspans rows = let
 
     dropColumns :: Integer -> [(a, Cell)] -> (Integer, [(a, Cell)])
     dropColumns n [] = (n, [])
-    dropColumns n cells@((_, Cell gridSpan _ _) : otherCells) =
+    dropColumns n cells@((_, Cell _ gridSpan _ _) : otherCells) =
       if n < gridSpan
       then (gridSpan - n, cells)
       else dropColumns (n - gridSpan) otherCells
@@ -324,7 +373,7 @@ leftBiasedMergeRunStyle a b = RunStyle
 type Extent = Maybe (Double, Double)
 
 data ParPart = PlainRun Run
-             | ChangedRuns TrackedChange [Run]
+             | ChangedRuns TrackedChange [ParPart]
              | CommentStart CommentId Author (Maybe CommentDate) [BodyPart]
              | CommentEnd CommentId
              | BookMark BookMarkId Anchor
@@ -427,7 +476,7 @@ archiveToDocument zf = do
 
 elemToBody :: NameSpaces -> Element -> D Body
 elemToBody ns element | isElem ns "w" "body" element =
-  fmap Body (mapD (elemToBodyPart ns) (elChildren element))
+  Body . addCaptioned <$> mapD (elemToBodyPart ns) (elChildren element)
 elemToBody _ _ = throwError WrongElem
 
 archiveToStyles :: Archive -> (CharStyleMap, ParStyleMap)
@@ -505,6 +554,10 @@ relElemToRelationship fp relType element | qName (elName element) == "Relationsh
     return $ Relationship relType relId target'
 relElemToRelationship _ _ _ = Nothing
 
+extractTarget :: Element -> Maybe Target
+extractTarget element = do (Relationship _ _ target) <- relElemToRelationship "word/" InDocument element
+                           return target
+
 filePathToRelationships :: Archive -> FilePath -> FilePath ->  [Relationship]
 filePathToRelationships ar docXmlPath fp
   | Just relType <- filePathToRelType fp docXmlPath
@@ -680,7 +733,16 @@ elemToCell ns element | isElem ns "w" "tc" element =
                          "restart" -> Just Restart
                          _ -> Nothing
     cellContents <- mapD (elemToBodyPart ns) (elChildren element)
-    return $ Cell (fromMaybe 1 gridSpan) vMerge cellContents
+    let align = case cellContents of -- take alignment from first paragraph
+                  Paragraph pstyle _ : _ ->
+                    case justification pstyle of
+                      Just JustifyBoth -> AlignLeft
+                      Just JustifyLeft -> AlignLeft
+                      Just JustifyRight -> AlignRight
+                      Just JustifyCenter -> AlignCenter
+                      Nothing -> AlignDefault
+                  _ -> AlignDefault
+    return $ Cell align (fromMaybe 1 gridSpan) vMerge cellContents
 elemToCell _ _ = throwError WrongElem
 
 testBitMask :: Text -> Int -> Bool
@@ -703,6 +765,24 @@ mkListItem parstyle numId lvl parparts = do
 pStyleIndentation :: ParagraphStyle -> Maybe ParIndentation
 pStyleIndentation style = (getParStyleField indent . pStyle) style
 
+addCaptioned :: [BodyPart] -> [BodyPart]
+addCaptioned [] = []
+addCaptioned (Paragraph parstyle parparts : x : xs)
+  | hasCaptionStyle parstyle
+  , isCaptionable x
+    = Captioned parstyle parparts x : addCaptioned xs
+addCaptioned (x : Paragraph parstyle parparts : xs)
+  | hasCaptionStyle parstyle
+  , not (pKeepNext parstyle)
+  , isCaptionable x
+    = Captioned parstyle parparts x : addCaptioned xs
+addCaptioned (x:xs) = x : addCaptioned xs
+
+isCaptionable :: BodyPart -> Bool
+isCaptionable (Paragraph _ [Drawing{}]) = True
+isCaptionable (Tbl{}) = True
+isCaptionable _ = False
+
 elemToBodyPart :: NameSpaces -> Element -> D BodyPart
 elemToBodyPart ns element
   | isElem ns "m" "oMathPara" element = do
@@ -714,50 +794,58 @@ elemToBodyPart ns element
 elemToBodyPart ns element
   | isElem ns "w" "p" element
   , Just (numId, lvl) <- getNumInfo ns element = do
+    lvlInfo <- lookupLevel numId lvl <$> asks envNumbering
     parstyle <- elemToParagraphStyle ns element
                 <$> asks envParStyles
                 <*> asks envNumbering
     parparts <- mconcat <$> mapD (elemToParPart ns) (elChildren element)
     case pHeading parstyle of
       Nothing -> mkListItem parstyle numId lvl parparts
-      Just _  -> do
-        return $ Paragraph parstyle parparts
+      Just (parstylename, lev)
+        -> return $ Heading lev parstylename parstyle numId lvl lvlInfo parparts
+elemToBodyPart ns element
+  | isElem ns "w" "p" element
+  , [Elem ppr] <- elContent element
+  , isElem ns "w" "pPr" ppr
+  , [Elem pbdr] <- elContent ppr
+  , isElem ns "w" "pBdr" pbdr
+    = return HRule
+      -- for this style of horizontal rule, see
+      -- https://support.microsoft.com/en-us/office/insert-a-horizontal-line-9bf172f6-5908-4791-9bb9-2c952197b1a9
+elemToBodyPart ns element -- pandoc-style horizontal rule
+  | isElem ns "w" "p" element
+  , [Elem r] <- elContent element
+  , isElem ns "w" "r" r
+  , [Elem pict] <- elContent r
+  , isElem ns "w" "pict" pict
+  , [Elem rect] <- elContent pict
+  , isElem ns "v" "rect" rect
+    = return HRule
 elemToBodyPart ns element
   | isElem ns "w" "p" element = do
       parstyle <- elemToParagraphStyle ns element
                   <$> asks envParStyles
                   <*> asks envNumbering
 
-      let hasCaptionStyle = elem "Caption" (pStyleId <$> pStyle parstyle)
+      let children =
+            (if hasCaptionStyle parstyle
+                then stripCaptionLabel
+                else id) (elChildren element)
 
-      let isTableNumberElt el@(Element name attribs _ _) =
-           (qName name == "fldSimple" &&
-             case lookupAttrBy ((== "instr") . qName) attribs of
-               Nothing -> False
-               Just instr -> "Table" `elem` T.words instr) ||
-           (qName name == "instrText" && "Table" `elem` T.words (strContent el))
-
-      let isTable = hasCaptionStyle &&
-                      isJust (filterChild isTableNumberElt element)
-
-      let stripOffLabel = dropWhile (not . isTableNumberElt)
-
-      let children = (if isTable
-                          then stripOffLabel
-                          else id) $ elChildren element
       parparts' <- mconcat <$> mapD (elemToParPart ns) children
       fldCharState <- gets stateFldCharState
       modify $ \st -> st {stateFldCharState = emptyFldCharContents fldCharState}
       -- Word uses list enumeration for numbered headings, so we only
       -- want to infer a list from the styles if it is NOT a heading.
-      let parparts = parparts' ++ (openFldCharsToParParts fldCharState)
+      let parparts = parparts' ++ openFldCharsToParParts fldCharState
       case pHeading parstyle of
         Nothing | Just (numId, lvl) <- pNumInfo parstyle -> do
                     mkListItem parstyle numId lvl parparts
-        _ -> if isTable
-                then return $ TblCaption parstyle parparts
-                else return $ Paragraph parstyle parparts
-
+        Just (parstylename, lev) -> do
+          let (numId, lvl) = fromMaybe ("","") $ pNumInfo parstyle
+          lvlInfo <- lookupLevel numId lvl <$> asks envNumbering
+          return $ Heading lev parstylename parstyle numId lvl lvlInfo parparts
+        Nothing -> return $ Paragraph parstyle parparts
 elemToBodyPart ns element
   | isElem ns "w" "tbl" element = do
     let tblProperties = findChildByName ns "w" "tblPr" element
@@ -767,6 +855,9 @@ elemToBodyPart ns element
         description = fromMaybe "" $ tblProperties
                        >>= findChildByName ns "w" "tblDescription"
                        >>= findAttrByName ns "w" "val"
+        mbstyle = tblProperties
+                       >>= findChildByName ns "w" "tblStyle"
+                       >>= findAttrByName ns "w" "val"
         grid' = case findChildByName ns "w" "tblGrid" element of
           Just g  -> elemToTblGrid ns g
           Nothing -> return []
@@ -779,7 +870,7 @@ elemToBodyPart ns element
     grid <- grid'
     tblLook <- tblLook'
     rows <- mapD (elemToRow ns) (elChildren element)
-    return $ Tbl (caption <> description) grid tblLook rows
+    return $ Tbl mbstyle (caption <> description) grid tblLook rows
 elemToBodyPart _ _ = throwError WrongElem
 
 lookupRelationship :: DocumentLocation -> RelId -> [Relationship] -> Maybe Target
@@ -854,6 +945,10 @@ example (omissions and my comments in brackets):
         <w:fldChar w:fldCharType="end"/>
       </w:r>
 
+Note that there may be mulitple w:instrText elements in a row.
+For example, you might first have ` XE "`, then `Kay, Alan`, then `"`.
+The texts in all of them should be concatenated before it is processed!
+
 So we do this in a number of steps. If we encounter the fldchar begin
 tag, we start open a fldchar state variable (see state above). We add
 the instrtext to it as FieldInfo. Then we close that and start adding
@@ -874,13 +969,15 @@ elemToParPart ns element
         _ | fldCharType == "begin" -> do
           modify $ \st -> st {stateFldCharState = FldCharOpen : fldCharState}
           return []
-        FldCharFieldInfo info : ancestors | fldCharType == "separate" -> do
+        FldCharFieldInfo t : ancestors | fldCharType == "separate" -> do
+          info <- eitherToD $ parseFieldInfo t
           modify $ \st -> st {stateFldCharState = FldCharContent info [] : ancestors}
           return []
-        -- Some fields have no content, since Pandoc doesn't understand any of those fields, we can just close it.
-        FldCharFieldInfo _ : ancestors | fldCharType == "end" -> do
+        -- Some fields have no content, e.g. index XE:
+        FldCharFieldInfo t : ancestors | fldCharType == "end" -> do
           modify $ \st -> st {stateFldCharState = ancestors}
-          return []
+          info <- eitherToD $ parseFieldInfo t
+          return [Field info []]
         [FldCharContent info children] | fldCharType == "end" -> do
           modify $ \st -> st {stateFldCharState = []}
           return [Field info $ reverse children]
@@ -895,8 +992,13 @@ elemToParPart ns element
       fldCharState <- gets stateFldCharState
       case fldCharState of
         FldCharOpen : ancestors -> do
-          info <- eitherToD $ parseFieldInfo $ strContent instrText
-          modify $ \st -> st {stateFldCharState = FldCharFieldInfo info : ancestors}
+          modify $ \st -> st {stateFldCharState =
+                               FldCharFieldInfo (strContent instrText) : ancestors}
+          return []
+        FldCharFieldInfo t : ancestors -> do
+          modify $ \st -> st {stateFldCharState =
+                               FldCharFieldInfo (t <> strContent instrText) :
+                                ancestors}
           return []
         _ -> return []
 {-
@@ -971,7 +1073,7 @@ elemToParPart' ns element
     return $ map PlainRun runs
 elemToParPart' ns element
   | Just change <- getTrackedChange ns element = do
-      runs <- mconcat <$> mapD (elemToRun ns) (elChildren element)
+      runs <- mconcat <$> mapD (elemToParPart' ns) (elChildren element)
       return [ChangedRuns change runs]
 elemToParPart' ns element
   | isElem ns "w" "bookmarkStart" element
@@ -1090,7 +1192,7 @@ elemToRun ns element
   | isElem ns "w" "r" element
   , Just altCont <- findChildByName ns "mc" "AlternateContent" element =
     do let choices = findChildrenByName ns "mc" "Choice" altCont
-           choiceChildren = map head $ filter (not . null) $ map elChildren choices
+           choiceChildren = concatMap (take 1 . elChildren) choices
        outputs <- mapD (childElemToRun ns) choiceChildren
        case outputs of
          r : _ -> return r
@@ -1158,6 +1260,14 @@ elemToParagraphStyle ns element sty numbering
       , numbered = case getNumInfo ns element of
           Just (numId, lvl) -> isJust $ lookupLevel numId lvl numbering
           Nothing -> isJust $ getParStyleField numInfo pStyle'
+      , justification =
+          case findChildByName ns "w" "jc" pPr >>= findAttrByName ns "w" "val" of
+            Nothing -> Nothing
+            Just "both" -> Just JustifyBoth
+            Just "center" -> Just JustifyCenter
+            Just "left" -> Just JustifyLeft
+            Just "right" -> Just JustifyRight
+            _ -> Nothing
       , indentation =
           getIndentation ns element
       , dropCap =
@@ -1176,6 +1286,7 @@ elemToParagraphStyle ns element sty numbering
                                   ) >>=
                       getTrackedChange ns
       , pBidi = checkOnOff ns pPr (elemName ns "w" "bidi")
+      , pKeepNext = isJust $ findChildByName ns "w" "keepNext" pPr
       }
   | otherwise = defaultParagraphStyle
 
@@ -1200,33 +1311,35 @@ elemToRunElem ns element
     case font of
       Nothing -> return $ TextRun str
       Just f  -> return . TextRun $
-                  T.map (\x -> fromMaybe x . getUnicode f . lowerFromPrivate $ x) str
+                   T.map (\c -> fromMaybe c (getFontChar f c)) str
   | isElem ns "w" "br" element = return LnBrk
   | isElem ns "w" "tab" element = return Tab
   | isElem ns "w" "softHyphen" element = return SoftHyphen
   | isElem ns "w" "noBreakHyphen" element = return NoBreakHyphen
   | isElem ns "w" "sym" element = return (getSymChar ns element)
   | otherwise = throwError WrongElem
-  where
-    lowerFromPrivate (ord -> c)
-      | c >= ord '\xF000' = chr $ c - ord '\xF000'
-      | otherwise = chr c
 
 -- The char attribute is a hex string
 getSymChar :: NameSpaces -> Element -> RunElem
 getSymChar ns element
-  | Just s <- lowerFromPrivate <$> getCodepoint
+  | Just s <- getCodepoint
   , Just font <- getFont =
     case readLitChar ("\\x" ++ T.unpack s) of
-         [(char, _)] -> TextRun . maybe "" T.singleton $ getUnicode font char
-         _           -> TextRun ""
+         [(ch, _)] ->
+              TextRun $ T.singleton $ fromMaybe ch $ getFontChar font ch
+         _ -> TextRun ""
   where
     getCodepoint = findAttrByName ns "w" "char" element
-    getFont = textToFont =<< findAttrByName ns "w" "font" element
-    lowerFromPrivate t | "F" `T.isPrefixOf` t = "0" <> T.drop 1 t
-                       | otherwise             = t
+    getFont = findAttrByName ns "w" "font" element >>= textToFont
 getSymChar _ _ = TextRun ""
 
+getFontChar :: Font -> Char -> Maybe Char
+getFontChar font ch = chr <$> M.lookup (font, point) symbolMap
+ where
+   point  -- sometimes F000 is added to put char in private range:
+      | ch >= '\xF000' = ord ch - 0xF000
+      | otherwise = ord ch
+
 elemToRunElems :: NameSpaces -> Element -> D [RunElem]
 elemToRunElems ns element
   |  isElem ns "w" "r" element
@@ -1234,9 +1347,9 @@ elemToRunElems ns element
        let qualName = elemName ns "w"
        let font = do
                     fontElem <- findElement (qualName "rFonts") element
-                    textToFont =<<
-                       foldr ((<|>) . (flip findAttr fontElem . qualName))
+                    foldr ((<|>) . (flip findAttr fontElem . qualName))
                          Nothing ["ascii", "hAnsi"]
+                      >>= textToFont
        local (setFont font) (mapD (elemToRunElem ns) (elChildren element))
 elemToRunElems _ _ = throwError WrongElem
 
@@ -1250,3 +1363,27 @@ findBlip el = do
   filterElementName (\(QName tag _ _) -> tag == "svgBlip") el `mplus` pure blip
  where
   a_ns = "http://schemas.openxmlformats.org/drawingml/2006/main"
+
+hasCaptionStyle :: ParagraphStyle -> Bool
+hasCaptionStyle parstyle = any (isCaptionStyleName . pStyleName) (pStyle parstyle)
+ where -- note that these are case insensitive:
+   isCaptionStyleName "caption" = True
+   isCaptionStyleName "table caption" = True
+   isCaptionStyleName "image caption" = True
+   isCaptionStyleName _ = False
+
+stripCaptionLabel :: [Element] -> [Element]
+stripCaptionLabel els =
+  if any isNumberElt els
+     then dropWhile (not . isNumberElt) els
+     else els
+  where
+    isNumberElt el@(Element name attribs _ _) =
+       (qName name == "fldSimple" &&
+             case lookupAttrBy ((== "instr") . qName) attribs of
+               Nothing -> False
+               Just instr -> "Table" `elem` T.words instr ||
+                             "Figure" `elem` T.words instr) ||
+       (qName name == "instrText" &&
+          let ws = T.words (strContent el)
+          in  ("Table" `elem` ws || "Figure" `elem` ws))
