@@ -67,6 +67,7 @@ import Text.Pandoc.Readers.LaTeX.SIunitx (siunitxCommands)
 import Text.Pandoc.Readers.LaTeX.Inline (acronymCommands, refCommands,
                                          nameCommands, charCommands,
                                          accentCommands,
+                                         miscCommands,
                                          biblatexInlineCommands,
                                          verbCommands, rawInlineOr,
                                          listingsLanguage)
@@ -331,6 +332,7 @@ unescapeURL = T.concat . go . T.splitOn "\\"
 inlineCommands :: PandocMonad m => M.Map Text (LP m Inlines)
 inlineCommands = M.unions
   [ accentCommands tok
+  , miscCommands
   , citationCommands inline
   , siunitxCommands tok
   , acronymCommands
@@ -343,6 +345,12 @@ inlineCommands = M.unions
   , biblatexInlineCommands tok
   , rest ]
  where
+  disableLigatures p = do
+    oldLigatures <- sLigatures <$> getState
+    updateState (\s -> s{ sLigatures = False })
+    res <- p
+    updateState (\s -> s{ sLigatures = oldLigatures })
+    pure res
   rest = M.fromList
     [ ("emph", extractSpaces emph <$> tok)
     , ("textit", extractSpaces emph <$> tok)
@@ -352,7 +360,7 @@ inlineCommands = M.unions
     , ("textmd", extractSpaces (spanWith ("",["medium"],[])) <$> tok)
     , ("textrm", extractSpaces (spanWith ("",["roman"],[])) <$> tok)
     , ("textup", extractSpaces (spanWith ("",["upright"],[])) <$> tok)
-    , ("texttt", formatCode nullAttr <$> tok)
+    , ("texttt", formatCode nullAttr <$> disableLigatures tok)
     , ("alert", skipopts >> spanWith ("",["alert"],[]) <$> tok) -- beamer
     , ("textsuperscript", extractSpaces superscript <$> tok)
     , ("textsubscript", extractSpaces subscript <$> tok)
@@ -386,20 +394,18 @@ inlineCommands = M.unions
     , ("lowercase", makeLowercase <$> tok)
     , ("thanks", skipopts >> note <$> grouped block)
     , ("footnote", skipopts >> footnote)
+    , ("newline", pure B.linebreak)
+    , ("linebreak", pure B.linebreak)
     , ("passthrough", fixPassthroughEscapes <$> tok)
     -- \passthrough macro used by latex writer
                            -- for listings
     , ("includegraphics", do options <- option [] keyvals
-                             src <- braced
-                             mkImage options .
-                               unescapeURL .
-                               removeDoubleQuotes $ untokenize src)
+                             src <- bracedFilename
+                             mkImage options . unescapeURL $ src)
     -- svg
     , ("includesvg",      do options <- option [] keyvals
-                             src <- braced
-                             mkImage options .
-                               unescapeURL .
-                               removeDoubleQuotes $ untokenize src)
+                             src <- bracedFilename
+                             mkImage options . unescapeURL $ src)
     -- hyperref
     , ("url", (\url -> linkWith ("",["uri"],[]) url "" (str url))
                         . unescapeURL . untokenize <$> bracedUrl)
@@ -438,6 +444,14 @@ inlineCommands = M.unions
     , ("today", today)
     ]
 
+bracedFilename :: PandocMonad m => LP m Text
+bracedFilename =
+  removeDoubleQuotes . T.strip . untokenize . filter (not . isComment) <$> braced
+
+isComment :: Tok -> Bool
+isComment (Tok _ Comment _) = True
+isComment _ = False
+
 today :: PandocMonad m => LP m Inlines
 today =
   text . T.pack . showGregorian . localDay . zonedTimeToLocalTime
@@ -628,6 +642,7 @@ inline = do
         do eatOneToken
            report $ ParsingUnescaped t pos
            return $ str t
+  ligatures <- sLigatures <$> getState
   case toktype of
     Comment     -> mempty <$ eatOneToken
     Spaces      -> space <$ eatOneToken
@@ -635,14 +650,19 @@ inline = do
     Word        -> str t <$ eatOneToken
     Symbol      ->
       case t of
-        "-"     -> eatOneToken *>
+        "-" | ligatures
+                -> eatOneToken *>
                     option (str "-") (symbol '-' *>
                       option (str "–") (str "—" <$ symbol '-'))
         "'"     -> eatOneToken *>
-                    option (str "’") (str  "”" <$ symbol '\'')
+                    option (str "’") (str  "”" <$ (guard ligatures *> symbol '\''))
         "~"     -> str "\160" <$ eatOneToken
-        "`"     -> doubleQuote <|> singleQuote <|> symbolAsString
-        "\""    -> doubleQuote <|> singleQuote <|> symbolAsString
+        "`" | ligatures
+                -> doubleQuote <|> singleQuote <|> (str "‘" <$ symbol '`')
+            | otherwise
+                -> str "‘" <$ symbol '`'
+        "\"" | ligatures
+                -> doubleQuote <|> singleQuote <|> symbolAsString
         "“"     -> doubleQuote <|> symbolAsString
         "‘"     -> singleQuote <|> symbolAsString
         "$"     -> dollarsMath <|> unescapedSymbolAsString
@@ -719,7 +739,7 @@ rawBlockOr name fallback = do
 doSubfile :: PandocMonad m => LP m Blocks
 doSubfile = do
   skipMany opt
-  f <- T.unpack . removeDoubleQuotes . T.strip . untokenize <$> braced
+  f <- T.unpack <$> bracedFilename
   oldToks <- getInput
   setInput $ TokStream False []
   insertIncluded (ensureExtension (/= "") ".tex" f)
@@ -737,7 +757,7 @@ include name = do
           _ -> const False
   skipMany opt
   fs <- map (T.unpack . removeDoubleQuotes . T.strip) . T.splitOn "," .
-         untokenize <$> braced
+         untokenize . filter (not . isComment) <$> braced
   mapM_ (insertIncluded . ensureExtension isAllowed ".tex") fs
   return mempty
 
@@ -745,7 +765,7 @@ usepackage :: (PandocMonad m, Monoid a) => LP m a
 usepackage = do
   skipMany opt
   fs <- map (T.unpack . removeDoubleQuotes . T.strip) . T.splitOn "," .
-         untokenize <$> braced
+           untokenize . filter (not . isComment) <$> braced
   let parsePackage f = do
         TokStream _ ts <- getIncludedToks (ensureExtension (== ".sty") ".sty" f)
         parseFromToks (do _ <- blocks
