@@ -173,6 +173,7 @@ data LaTeXState = LaTeXState{ sOptions       :: ReaderOptions
                             , sFileContents  :: M.Map Text Text
                             , sEnableWithRaw :: Bool
                             , sRawTokens     :: IntMap.IntMap [Tok]
+                            , sLigatures     :: Bool
                             }
      deriving Show
 
@@ -201,6 +202,7 @@ defaultLaTeXState = LaTeXState{ sOptions       = def
                               , sFileContents  = M.empty
                               , sEnableWithRaw = True
                               , sRawTokens     = IntMap.empty
+                              , sLigatures     = True
                               }
 
 instance PandocMonad m => HasQuoteContext LaTeXState m where
@@ -934,6 +936,7 @@ withRaw parser = do
 
 keyval :: PandocMonad m => LP m (Text, Text)
 keyval = try $ do
+  sp
   key <- untokenize <$> many1 (notFollowedBy (symbol '=') >>
                          (symbol '-' <|> symbol '_' <|> satisfyTok isWordTok))
   sp
@@ -952,6 +955,7 @@ keyval = try $ do
                                 Tok _ Symbol "{" -> False
                                 Tok _ Symbol "}" -> False
                                 _                -> True)))))
+  sp
   optional (symbol ',')
   sp
   return (key, T.strip val)
