@@ -105,8 +105,10 @@ pInline = try $ do
           B.math . writeTeX <$> pMathMany (Seq.singleton res)
     Elt name@(Identifier tname) pos fields -> do
       labs <- sLabels <$> getState
-      labelTarget <- (do VLabel t <- getField "target" fields
-                         True <$ guard (t `elem` labs))
+      labelTarget <- (do result <- getField "target" fields
+                         case result of
+                           VLabel t | t `elem` labs -> pure True
+                           _ -> pure False)
                   <|> pure False
       if tname == "ref" && not labelTarget
          then do
@@ -233,10 +235,11 @@ blockHandlers = M.fromList
   ,("quote", \_ fields -> do
       getField "block" fields >>= guard
       body <- getField "body" fields >>= pWithContents pBlocks
-      attribution <-
-        ((\x -> B.para ("\x2104\xa0" <> x)) <$>
-          (getField "attribution" fields >>= pWithContents pInlines))
-        <|> pure mempty
+      attribution' <- getField "attribution" fields
+      attribution <- if attribution' == mempty
+                        then pure mempty
+                        else (\x -> B.para ("\x2014\xa0" <> x)) <$>
+                              (pWithContents pInlines attribution')
       pure $ B.blockQuote $ body <> attribution)
   ,("list", \_ fields -> do
       children <- V.toList <$> getField "children" fields
@@ -292,7 +295,7 @@ blockHandlers = M.fromList
       pure $ B.codeBlockWith attr txt)
   ,("parbreak", \_ _ -> pure mempty)
   ,("block", \mbident fields ->
-      B.divWith (fromMaybe "" mbident, [], [])
+      maybe id (\ident -> B.divWith (ident, [], [])) mbident
         <$> (getField "body" fields >>= pWithContents pBlocks))
   ,("place", \_ fields -> do
       ignored "parameters of place"
@@ -363,6 +366,8 @@ blockHandlers = M.fromList
       pure $ B.plain . B.text . mconcat . map toNum $ V.toList nums)
   ,("footnote.entry", \_ fields ->
       getField "body" fields >>= pWithContents pBlocks)
+  ,("pad", \_ fields ->  -- ignore paddingy
+      getField "body" fields >>= pWithContents pBlocks)
   ]
 
 inlineHandlers :: PandocMonad m =>
@@ -461,7 +466,7 @@ inlineHandlers = M.fromList
                 (B.fromList . blocksToInlines . B.toList <$> pBlocks) body
       pure $ B.link src "" description)
   ,("image", \_ fields -> do
-      path <- getField "path" fields
+      path <- getField "source" fields <|> getField "path" fields
       alt <- (B.text <$> getField "alt" fields) `mplus` pure mempty
       (mbwidth :: Maybe Text) <-
         fmap (renderLength False) <$> getField "width" fields
@@ -497,6 +502,11 @@ inlineHandlers = M.fromList
       body <- getField "body" fields
       display <- getField "block" fields
       (if display then B.displayMath else B.math) . writeTeX <$> pMathMany body)
+  ,("pad", \_ fields ->  -- ignore paddingy
+      getField "body" fields >>= pWithContents pInlines)
+  ,("block", \mbident fields ->
+      maybe id (\ident -> B.spanWith (ident, [], [])) mbident
+        <$> (getField "body" fields >>= pWithContents pInlines))
   ]
 
 getInlineBody :: PandocMonad m => M.Map Identifier Val -> P m (Seq Content)
@@ -513,9 +523,12 @@ parbreaksToLinebreaks =
    isParbreak _ = False
 
 pPara :: PandocMonad m => P m B.Blocks
-pPara =
-  B.para . B.trimInlines . collapseAdjacentCites . mconcat
-    <$> (many1 pInline <* optional pParBreak)
+pPara = do
+  ils <- B.trimInlines . collapseAdjacentCites . mconcat <$> many1 pInline
+  optional pParBreak
+  pure $ if ils == mempty
+         then mempty
+         else B.para ils
 
 pParBreak :: PandocMonad m => P m ()
 pParBreak =
@@ -537,7 +550,8 @@ pWithContents pa cs = try $ do
 
 pInlines :: PandocMonad m => P m B.Inlines
 pInlines =
-  collapseAdjacentCites . mconcat <$> many pInline
+  mappend <$> (collapseAdjacentCites . mconcat <$> many pInline)
+          <*> ((B.softbreak <$ pParBreak) <|> pure mempty)
 
 collapseAdjacentCites :: B.Inlines -> B.Inlines
 collapseAdjacentCites = B.fromList . foldr go [] . B.toList
