@@ -4,7 +4,7 @@
 {-# LANGUAGE TupleSections     #-}
 {- |
    Module      : Text.Pandoc.SelfContained
-   Copyright   : Copyright (C) 2011-2023 John MacFarlane
+   Copyright   : Copyright (C) 2011-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -19,12 +19,12 @@ module Text.Pandoc.SelfContained ( makeDataURI, makeSelfContained ) where
 import Codec.Compression.GZip as Gzip
 import Control.Applicative ((<|>))
 import Data.ByteString (ByteString)
-import Data.ByteString.Base64 (encodeBase64)
+import Data.ByteString.Base64 (encode)
 import qualified Data.ByteString.Char8 as B
 import qualified Data.ByteString.Lazy as L
 import qualified Data.Text as T
 import Data.Char (isAlphaNum, isAscii)
-import Data.Digest.Pure.SHA (sha1, showDigest)
+import Crypto.Hash (hashWith, SHA1(SHA1))
 import Network.URI (escapeURIString)
 import System.FilePath (takeDirectory, takeExtension, (</>))
 import Text.HTML.TagSoup
@@ -52,8 +52,11 @@ makeDataURI :: (MimeType, ByteString) -> T.Text
 makeDataURI (mime, raw) =
   if textual
      then "data:" <> mime' <> "," <> T.pack (escapeURIString isOk (toString raw))
-     else "data:" <> mime' <> ";base64," <> encodeBase64 raw
+     else "data:" <> mime' <> ";base64," <> toText (encode raw')
   where textual = "text/" `T.isPrefixOf` mime
+        raw' = if "+xml" `T.isSuffixOf` mime
+                  then B.filter (/= '\r') raw  -- strip off CRs
+                  else raw
         mime' = if textual && T.any (== ';') mime
                    then mime <> ";charset=utf-8"
                    else mime  -- mime type already has charset
@@ -87,7 +90,7 @@ convertTags (t@(TagOpen "style" _):ts) =
       ((t:xs') ++) <$> convertTags rest
 convertTags (t@(TagOpen "script" as):tc@(TagClose "script"):ts) =
   case fromAttrib "src" t of
-       ""  -> (t:) <$> convertTags ts
+       ""  -> ([t, tc] ++) <$> convertTags ts
        src -> do
            let typeAttr = fromAttrib "type" t
            res <- getData typeAttr src
@@ -102,7 +105,9 @@ convertTags (t@(TagOpen "script" as):tc@(TagClose "script"):ts) =
                      "application/x-javascript" `T.isPrefixOf` mime) &&
                      not ("</script" `B.isInfixOf` bs) ->
                      return $
-                       TagOpen "script" [("type", typeAttr)|not (T.null typeAttr)]
+                       TagOpen "script" [(k,v) | (k,v) <- as
+                                               , k == "type" ||
+                                                 "data-" `T.isPrefixOf` k]
                        : TagText (toText bs)
                        : TagClose "script"
                        : rest
@@ -124,7 +129,8 @@ convertTags (t@(TagOpen "link" as):ts) =
                      (("href",dataUri) : [(x,y) | (x,y) <- as, x /= "href"]) :
                      rest
                 Fetched (mime, bs)
-                  | "text/css" `T.isPrefixOf` mime
+                  | ("text/css" `T.isPrefixOf` mime ||
+                      fromAttrib "rel" t == "stylesheet")
                     && T.null (fromAttrib "media" t)
                     && not ("</" `B.isInfixOf` bs) -> do
                       rest <- convertTags $
@@ -145,8 +151,12 @@ convertTags (t@(TagOpen "link" as):ts) =
 convertTags (t@(TagOpen tagname as):ts)
   | any (isSourceAttribute tagname) as
      = do
-       as' <- mapM processAttribute as
-       let attrs = rights as'
+       let inlineSvgs = tagname == "img" &&
+                        case T.words <$> lookup "class" as of
+                          Nothing -> False
+                          Just cs -> "inline-svg" `elem` cs
+       as' <- mapM (processAttribute inlineSvgs) as
+       let attrs = addRole "img" $ addAriaLabel $ rights as'
        let svgContents = lefts as'
        rest <- convertTags ts
        case svgContents of
@@ -159,9 +169,12 @@ convertTags (t@(TagOpen tagname as):ts)
              svgmap <- gets svgMap
              case M.lookup hash svgmap of
                Just (svgid, svgattrs) -> do
-                 let attrs' = combineSvgAttrs svgattrs attrs
+                 let attrs' = [(k,v) | (k,v) <- combineSvgAttrs svgattrs attrs
+                                     , k /= "id"]
                  return $ TagOpen "svg" attrs' :
-                          TagOpen "use" [("href", "#" <> svgid)] :
+                          TagOpen "use" [("href", "#" <> svgid),
+                                         ("width", "100%"),
+                                         ("height", "100%")] :
                           TagClose "use" :
                           TagClose "svg" :
                           rest'
@@ -172,22 +185,41 @@ convertTags (t@(TagOpen tagname as):ts)
                       let svgid = case lookup "id" attrs' of
                                      Just id' -> id'
                                      Nothing -> "svg_" <> hash
-                      let attrs'' = [(k,v) | (k,v) <- attrs', k /= "id"]
+                      let attrs'' = ("id", svgid) :
+                                    [(k,v) | (k,v) <- attrs', k /= "id"]
                       modify $ \st ->
                         st{ svgMap = M.insert hash (svgid, attrs'') (svgMap st) }
-                      return $ TagOpen "svg" attrs'' : tags' ++ rest'
+                      let fixUrl x =
+                            case T.breakOn "url(#" x of
+                              (_,"") -> x
+                              (before, after) -> before <>
+                                  "url(#" <> svgid <> "_" <> T.drop 5 after
+                      let addIdPrefix ("id", x) = ("id", svgid <> "_" <> x)
+                          addIdPrefix (k, x)
+                           | k == "xlink:href" || k == "href" =
+                            case T.uncons x of
+                              Just ('#', x') -> (k, "#" <> svgid <> "_" <> x')
+                              _ -> (k, x)
+                          -- this clause handles things like
+                          -- style="fill:url(#radialGradient46);stroke:none",
+                          -- adding the svg id prefix to the anchor:
+                          addIdPrefix (k, x) = (k, fixUrl x)
+                      let ensureUniqueId (TagOpen tname ats) =
+                            TagOpen tname (map addIdPrefix ats)
+                          ensureUniqueId x = x
+                      return $ TagOpen "svg" attrs'' :
+                                 map ensureUniqueId tags' ++ rest'
                     _ -> return $ TagOpen tagname attrs : rest
-  where processAttribute (x,y) =
+  where processAttribute inlineSvgs (x,y) =
            if isSourceAttribute tagname (x,y)
               then do
                 res <- getData (fromAttrib "type" t) y
                 case res of
                   AlreadyDataURI enc -> return $ Right (x, enc)
-                  Fetched ("image/svg+xml", bs) -> do
+                  Fetched ("image/svg+xml", bs) | inlineSvgs -> do
                     -- we filter CR in the hash to ensure that Windows
                     -- and non-Windows tests agree:
-                    let hash = T.pack $ take 20 $ showDigest $
-                                        sha1 $ L.fromStrict
+                    let hash = T.pack $ take 20 $ show $ hashWith SHA1
                                              $ B.filter (/='\r') bs
                     return $ Left (hash, getSvgTags (toText bs))
                   Fetched (mt,bs) -> return $ Right (x, makeDataURI (mt,bs))
@@ -196,6 +228,20 @@ convertTags (t@(TagOpen tagname as):ts)
 
 convertTags (t:ts) = (t:) <$> convertTags ts
 
+addRole :: T.Text -> [(T.Text, T.Text)] -> [(T.Text, T.Text)]
+addRole role attrs =
+  case lookup "role" attrs of
+    Nothing -> ("role", role) : attrs
+    Just _ -> attrs
+
+addAriaLabel :: [(T.Text, T.Text)] -> [(T.Text, T.Text)]
+addAriaLabel attrs =
+  case lookup "aria-label" attrs of
+    Just _ -> attrs
+    Nothing -> case lookup "alt" attrs of
+                 Just alt -> ("aria-label", alt) : attrs
+                 Nothing -> attrs
+
 -- we want to drop spaces, <?xml>, and comments before <svg>
 -- and anything after </svg>:
 getSvgTags :: T.Text -> [Tag T.Text]
@@ -221,10 +267,13 @@ combineSvgAttrs svgAttrs imgAttrs =
   dropPointZero t = case T.stripSuffix ".0" t of
                        Nothing -> t
                        Just t' -> t'
-  combinedAttrs = imgAttrs ++
+  combinedAttrs =
+    [(k, v) | (k, v) <- imgAttrs
+            , k /= "class"] ++
     [(k, v) | (k, v) <- svgAttrs
             , isNothing (lookup k imgAttrs)
-            , k `notElem` ["xmlns", "xmlns:xlink", "version"]]
+            , k `notElem` ["xmlns", "xmlns:xlink", "version", "class"]] ++
+    mergedClasses
   parseViewBox t =
     case map (safeRead . addZero) $ T.words t of
       [Just llx, Just lly, Just urx, Just ury] -> Just (llx, lly, urx, ury)
@@ -237,6 +286,9 @@ combineSvgAttrs svgAttrs imgAttrs =
         lookup "viewBox" svgAttrs >>= parseViewBox
   (mbHeight :: Maybe Int) = lookup "height" combinedAttrs >>= safeRead
   (mbWidth :: Maybe Int) = lookup "width" combinedAttrs >>= safeRead
+  mergedClasses = case (lookup "class" imgAttrs, lookup "class" svgAttrs) of
+                    (Just c1, Just c2) -> [("class", c1 <> " " <> c2)]
+                    _ -> []
 
 cssURLs :: PandocMonad m
         => FilePath -> ByteString -> m ByteString
