@@ -33,14 +33,13 @@ import Control.Monad
 import Data.Containers.ListUtils (nubOrd)
 import Data.Char (isDigit)
 import Data.List (intersperse, (\\))
-import Data.Maybe (catMaybes, fromMaybe, isJust, mapMaybe, isNothing,
-                   maybeToList)
+import Data.Maybe (catMaybes, fromMaybe, isJust, mapMaybe, isNothing)
 import Data.Monoid (Any (..))
 import Data.Text (Text)
 import qualified Data.Text as T
 import Network.URI (unEscapeString)
 import Text.DocTemplates (FromContext(lookupContext), Val(..), renderTemplate)
-import Text.Collate.Lang (renderLang, Lang(langLanguage))
+import Text.Collate.Lang (renderLang)
 import Text.Pandoc.Class.PandocMonad (PandocMonad, report, toLang)
 import Text.Pandoc.Definition
 import Text.Pandoc.Highlighting (formatLaTeXBlock, formatLaTeXInline, highlight,
@@ -275,11 +274,6 @@ pandocToLaTeX options (Pandoc meta blocks) = do
                 nubOrd . catMaybes .
                 filter (/= babelLang)
                 $ map toBabel docLangs))
-        $ defField "selnolig-langs"
-             (literal . T.intercalate "," $
-               let langs = docLangs ++ maybeToList mblang
-               in (["english" | any ((== "en") . langLanguage) langs] ++
-                   ["german" | any ((== "de") . langLanguage) langs]))
         $ defField "latex-dir-rtl"
            ((render Nothing <$> getField "dir" context) ==
                Just ("rtl" :: Text)) context
@@ -1083,12 +1077,20 @@ inlineToLaTeX il@(Image _ _ (src, _))
   | Just _ <- T.stripPrefix "data:" src = do
       report $ InlineNotRendered il
       return empty
-inlineToLaTeX (Image attr@(_,_,kvs) _ (source, _)) = do
+inlineToLaTeX (Image attr@(_,_,kvs) description (source, _)) = do
   setEmptyLine False
   let isSVG = ".svg" `T.isSuffixOf` source || ".SVG" `T.isSuffixOf` source
   modify $ \s -> s{ stGraphics = True
                   , stSVG = stSVG s || isSVG }
   opts <- gets stOptions
+  mbalt <- if isSVG
+              then pure Nothing
+              else case lookup "alt" kvs of
+                     Just x -> Just <$> stringToLaTeX TextString x
+                     Nothing
+                       | null description -> pure Nothing
+                       | otherwise -> Just <$> stringToLaTeX TextString
+                                                  (stringify description)
   let showDim dir = let d = text (show dir) <> "="
                     in case dimension dir attr of
                          Just (Pixel a)   ->
@@ -1114,6 +1116,7 @@ inlineToLaTeX (Image attr@(_,_,kvs) _ (source, _)) = do
                   _ -> ["keepaspectratio"]) <>
                 maybe [] (\x -> ["page=" <> literal x]) (lookup "page" kvs) <>
                 maybe [] (\x -> ["trim=" <> literal x]) (lookup "trim" kvs) <>
+                maybe [] (\x -> ["alt=" <> braces (literal x)]) mbalt <>
                 maybe [] (const ["clip"]) (lookup "clip" kvs)
       options = if null optList
                    then empty
