@@ -1,7 +1,7 @@
 {-# LANGUAGE OverloadedStrings   #-}
 {- |
    Module      : Text.Pandoc.Writers.LaTeX.Util
-   Copyright   : Copyright (C) 2006-2023 John MacFarlane
+   Copyright   : Copyright (C) 2006-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -108,11 +108,11 @@ stringToLaTeX context zs = do
          '}' -> emits "\\}"
          '?' | ligatures ->  -- avoid ?` ligature
            case xs of
-             '`':_ -> emits "?{}"
+             '`':_ -> emits "?{\\kern0pt}" -- se #10610
              _     -> emitc x
          '!' | ligatures ->  -- avoid !` ligature
            case xs of
-             '`':_ -> emits "!{}"
+             '`':_ -> emits "!{\\kern0pt}"
              _     -> emitc x
          '`' | ctx == CodeString -> emitcseq "\\textasciigrave"
          '$' -> emits "\\$"
@@ -235,7 +235,7 @@ wrapDiv (_,classes,kvs) t = do
                                w = maybe "0.48" fromPct (lookup "width" kvs)
                            in  inCmd "begin" "column" <>
                                valign <>
-                               braces (literal w <> "\\textwidth")
+                               braces (literal w <> "\\linewidth")
                                $$ contents
                                $$ inCmd "end" "column"
                     else id
@@ -260,8 +260,14 @@ wrapDiv (_,classes,kvs) t = do
 hypertarget :: PandocMonad m => Text -> LW m (Doc Text)
 hypertarget "" = return mempty
 hypertarget ident = do
-  label <- labelFor ident
-  return $ text "\\phantomsection" <> label
+  inHeading <- gets stInHeading
+  if inHeading
+     then do -- see #9209 (these cases should be rare)
+      ref <- literal <$> toLabel ident
+      return $ text "\\protect\\hypertarget" <> braces ref <> "{}"
+     else do
+      label <- labelFor ident
+      return $ text "\\protect\\phantomsection" <> label
 
 labelFor :: PandocMonad m => Text -> LW m (Doc Text)
 labelFor ""    = return empty
