@@ -2,7 +2,7 @@
 {-# LANGUAGE OverloadedStrings   #-}
 {- |
    Module      : Text.Pandoc.Writers.RTF
-   Copyright   : Copyright (C) 2006-2022 John MacFarlane
+   Copyright   : Copyright (C) 2006-2024 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -134,7 +134,7 @@ handleUnicode = T.concatMap $ \c ->
   where
     surrogate x = not (   (0x0000 <= ord x && ord x <= 0xd7ff)
                        || (0xe000 <= ord x && ord x <= 0xffff) )
-    enc x = "\\u" <> tshow (ord x) <> "?"
+    enc x = "\\u" <> tshow (ord x) <> " ?"
 
 -- | Escape special characters.
 escapeSpecial :: Text -> Text
@@ -230,7 +230,6 @@ blockToRTF :: PandocMonad m
            -> Alignment -- ^ alignment
            -> Block     -- ^ block to convert
            -> m Text
-blockToRTF _ _ Null = return ""
 blockToRTF indent alignment (Div _ bs) =
   blocksToRTF indent alignment bs
 blockToRTF indent alignment (Plain lst) =
@@ -270,6 +269,8 @@ blockToRTF indent alignment (Table _ blkCapt specs thead tbody tfoot) = do
                 else tableRowToRTF True indent aligns sizes headers
   rows' <- T.concat <$> mapM (tableRowToRTF False indent aligns sizes) rows
   return $ header' <> rows' <> rtfPar indent 0 alignment caption'
+blockToRTF indent alignment (Figure attr capt body) =
+  blockToRTF indent alignment $ figureDiv attr capt body
 
 tableRowToRTF :: PandocMonad m
               => Bool -> Int -> [Alignment] -> [Double] -> [[Block]] -> m Text
@@ -280,8 +281,9 @@ tableRowToRTF header indent aligns sizes' cols = do
                  else sizes'
   columns <- T.concat <$>
      zipWithM (tableItemToRTF indent) aligns cols
-  let rightEdges = tail $ scanl (\sofar new -> sofar + floor (new * totalTwips))
-                                (0 :: Integer) sizes
+  let rightEdges = drop 1 $
+                    scanl (\sofar new -> sofar + floor (new * totalTwips))
+                          (0 :: Integer) sizes
   let cellDefs = map (\edge -> (if header
                                    then "\\clbrdrb\\brdrs"
                                    else "") <> "\\cellx" <> tshow edge)
