@@ -29,8 +29,7 @@ import Network.URI (unEscapeString)
 import qualified Data.Text as T
 import Control.Monad.State ( StateT, evalStateT, gets, modify )
 import Text.Pandoc.Writers.Shared ( metaToContext, defField, resetField,
-                                    lookupMetaString,
-                                    isOrderedListMarker )
+                                    lookupMetaString )
 import Text.Pandoc.Shared (isTightList, orderedListMarkers, tshow)
 import Text.Pandoc.Writers.Math (convertMath)
 import qualified Text.TeXMath as TM
@@ -39,7 +38,7 @@ import Text.DocTemplates (renderTemplate)
 import Text.Pandoc.Extensions (Extension(..))
 import Text.Collate.Lang (Lang(..), parseLang)
 import Text.Printf (printf)
-import Data.Char (isAlphaNum)
+import Data.Char (isAlphaNum, isDigit)
 import Data.Maybe (fromMaybe)
 
 -- | Convert Pandoc to Typst.
@@ -86,6 +85,7 @@ pandocToTypst options (Pandoc meta blocks) = do
                         Right l ->
                           resetField "lang" (langLanguage l) .
                           maybe id (resetField "region") (langRegion l))
+              $ defField "csl" (lookupMetaString "citation-style" meta) -- #10661
               $ defField "smart" (isEnabled Ext_smart options)
               $ defField "toc-depth" (tshow $ writerTOCDepth options)
               $ defField "figure-caption-position"
@@ -147,8 +147,12 @@ blocksToTypst blocks = vcat <$> mapM blockToTypst blocks
 blockToTypst :: PandocMonad m => Block -> TW m (Doc Text)
 blockToTypst block =
   case block of
-    Plain inlines -> inlinesToTypst inlines
-    Para inlines -> ($$ blankline) <$> inlinesToTypst inlines
+    Plain inlines -> do
+      opts <- gets stOptions
+      inlinesToTypst (addLineStartEscapes opts inlines)
+    Para inlines -> do
+      opts <- gets stOptions
+      ($$ blankline) <$> inlinesToTypst (addLineStartEscapes opts inlines)
     Header level (ident,cls,_) inlines -> do
       contents <- inlinesToTypst inlines
       let lab = toLabel FreestandingLabel ident
@@ -380,9 +384,12 @@ inlineToTypst inline =
       case res of
           Left il -> inlineToTypst il
           Right r ->
-            case mathType of
-              InlineMath -> return $ "$" <> literal r <> "$"
-              DisplayMath -> return $ "$ " <> literal r <> " $"
+            (case extractLabel str of -- #10805
+              Nothing -> id
+              Just lab -> (<> (toLabel FreestandingLabel lab))) <$>
+             case mathType of
+               InlineMath -> return $ "$" <> literal r <> "$"
+               DisplayMath -> return $ "$ " <> literal r <> " $"
     Code (_,cls,_) code -> return $
       case cls of
         (lang:_) -> "#raw(lang:" <> doubleQuoted lang <>
@@ -401,14 +408,15 @@ inlineToTypst inline =
     Superscript inlines -> textstyle "#super" inlines
     Subscript inlines -> textstyle "#sub" inlines
     SmallCaps inlines -> textstyle "#smallcaps" inlines
-    Span (ident,_,kvs) inlines -> do
+    Span (ident,cls,kvs) inlines -> do
       let lab = toLabel FreestandingLabel ident
       let (_, typstTextAttrs) = pickTypstAttrs kvs
-      case typstTextAttrs of
-        [] -> (<> lab) <$> inlinesToTypst inlines
-        _ -> do
-          contents <- inlinesToTypst inlines
-          return $ toTypstTextElement typstTextAttrs contents <> lab
+      contents <- inlinesToTypst inlines
+      let addHl x = "#highlight" <> brackets x
+      return $ (if "mark" `elem` cls
+                   then addHl
+                   else id)
+               (toTypstTextElement typstTextAttrs contents) <> lab
     Quoted quoteType inlines -> do
       opts <- gets stOptions
       let smart = isEnabled Ext_smart opts
@@ -474,16 +482,28 @@ mkImage opts useBox src attr
     | otherwise = "image" <> parens (doubleQuoted src' <> dimAttrs)
 
 textstyle :: PandocMonad m => Doc Text -> [Inline] -> TW m (Doc Text)
-textstyle s inlines =
-  (<> endCode) . (s <>) . brackets . addEscape <$> inlinesToTypst inlines
+textstyle s inlines = do
+  opts <-  gets stOptions
+  (<> endCode) . (s <>) . brackets
+    <$> inlinesToTypst (addLineStartEscapes opts inlines)
+
+addLineStartEscapes :: WriterOptions -> [Inline] -> [Inline]
+addLineStartEscapes opts = go True
  where
-   addEscape =
-     case inlines of
-       (Str t : _)
-         | isOrderedListMarker t -> ("\\" <>)
-         | Just (c, _) <- T.uncons t
-         , needsEscapeAtLineStart c -> ("\\" <>)
-       _ -> id
+   go True (Str t : xs)
+        | isOrderedListMarker t = RawInline "typst" "\\" : Str t : go False xs
+        | Just (c, t') <- T.uncons t
+        , needsEscapeAtLineStart c
+        , T.null t' = RawInline "typst" "\\" : Str t : go False xs
+   go _ (SoftBreak : xs)
+        | writerWrapText opts == WrapPreserve = SoftBreak : go True xs
+   go _ (LineBreak : xs) = LineBreak : go True xs
+   go _ (x : xs) = x : go False xs
+   go _ [] = []
+
+isOrderedListMarker :: Text -> Bool
+isOrderedListMarker t = not (T.null ds) && rest == "."
+  where (ds, rest) = T.span isDigit t
 
 escapeTypst :: Bool -> EscapeContext -> Text -> Doc Text
 escapeTypst smart context t =
@@ -588,3 +608,10 @@ doubleQuoted = doubleQuotes . literal . escape
 
 endCode :: Doc Text
 endCode = beforeNonBlank ";"
+
+extractLabel :: Text -> Maybe Text
+extractLabel = go . T.unpack
+ where
+   go [] = Nothing
+   go ('\\':'l':'a':'b':'e':'l':'{':xs) = Just (T.pack (takeWhile (/='}') xs))
+   go (_:xs) = go xs
